<?php

namespace App\Service;

use App\Entity\Kitchen;
use App\Entity\Restaurant;
use App\Entity\RestaurantType;
use App\Mapper\RestaurantMapper;
use App\Model\KitchenType;
use App\Model\Pagination;
use App\Model\RestaurantFilterVariants;
use App\Model\RestaurantList;
use App\Model\RestaurantListingElement;
use App\Model\RestaurantType as RestaurantTypeModel;
use App\Repository\Interface\KitchenRepositoryInterface;
use App\Repository\Interface\RestaurantRepositoryInterface;
use App\Repository\Interface\RestaurantTypeRepositoryInterface;
use Ramsey\Collection\Collection;

class RestaurantService
{
    public function __construct(
        private RestaurantRepositoryInterface $restaurantRepository,
        private RestaurantTypeRepositoryInterface $restaurantTypeRepository,
        private KitchenRepositoryInterface $kitchenRepository,
    ) {}

    public function getRestaurants(): RestaurantList
    {
        $restaurants = $this->restaurantRepository->getAll();
        $restaurantTypes = $this->restaurantTypeRepository->getAll();
        $kitchens = $this->kitchenRepository->getAll();

        return new RestaurantList(
            new Pagination(1, floor(count($restaurants)) / 5, 16),
            new Collection(RestaurantListingElement::class, array_map(
            function (Restaurant $restaurant) {
                return RestaurantMapper::mapToListElement($restaurant);
            }, $restaurants)),
            new RestaurantFilterVariants(
                new Collection(
                    RestaurantTypeModel::class, array_map(
                        function (RestaurantType $restaurantType) {
                            return RestaurantMapper::mapToRestaurantType($restaurantType);
                        }, $restaurantTypes
                    )
                ),
                new Collection(
                    KitchenType::class, array_map(
                        function (Kitchen $kitchen) {
                            return RestaurantMapper::mapToKitchenType($kitchen);
                        }, $kitchens
                    ),
                )
            )
        );
    }
}